@@ -20,6 +20,7 @@ from cosmos_rl.utils.parallelism import ParallelDims
 from cosmos_rl.utils.logging import logger
 from cosmos_rl.policy.config import Config as CosmosConfig
 import cosmos_rl.utils.util as util
+from cosmos_rl.utils.constant import COSMOS_HF_MODEL_TYPES
 import torch
 from transformers import AutoConfig
 from cosmos_rl.dispatcher.data.packer import DataPacker
@@ -380,9 +381,15 @@ class ModelRegistry:
         hf_config = util.retry(AutoConfig.from_pretrained)(
             model_name_or_path, trust_remote_code=True
         )
-        if hf_config.model_type not in ModelRegistry._MODEL_REGISTRY:
-            raise ValueError(f"Model {hf_config.model_type} not supported.")
-        model_cls = ModelRegistry._MODEL_REGISTRY[hf_config.model_type]
+        model_type = hf_config.model_type
+        is_supported_model_type = model_type in ModelRegistry._MODEL_REGISTRY
+        if not is_supported_model_type:
+            logger.info(
+                f"Model type {hf_config.model_type} not registered, using {COSMOS_HF_MODEL_TYPES} instead."
+            )
+            model_type = COSMOS_HF_MODEL_TYPES
+
+        model_cls = ModelRegistry._MODEL_REGISTRY[model_type]
 
         with torch.device("meta"):
             with util.cosmos_default_dtype(
@@ -423,7 +430,8 @@ class WeightMapper(ABC):
 
     @abstractmethod
     def rollout_prepare_recv(
-        self, vllm_model: Any
+        self,
+        vllm_model: Any,
     ) -> Tuple[Dict[str, torch.Tensor], List[List[Tuple[str, int]]]]:
         """
         Rollout prepare recv list for P2R weight sync:
