@@ -17,6 +17,7 @@ import msgpack
 import torch
 from cosmos_rl.utils.parallelism import ParallelDims
 from typing import Dict, List, Tuple, Callable, Any, Optional, Union
+from cosmos_rl.utils.constant import COSMOS_HF_MODEL_TYPES
 from cosmos_rl.policy.model.base import WeightMapper
 from cosmos_rl.utils.logging import logger
 from vllm.model_executor.layers.linear import (
@@ -696,6 +697,7 @@ class ParallelTopoMapper:
             dims_rank_info = {}
             if not is_dist_tensor:
                 dims_map = {}
+                global_shape = tuple(param.shape)
             else:
                 dims_map = {}
                 global_shape = tuple(param.shape)
@@ -836,10 +838,7 @@ class ParallelTopoMapper:
                 continue
             dims_map = {}
             if isinstance(part, (QKVParallelLinear)):
-                output_dim = getattr(param, "output_dim", None)
-                assert (
-                    output_dim is not None
-                ), f"QKVParallelLinear {param_name} has no output_dim attribute."
+                output_dim = getattr(param, "output_dim", 0)
                 dims_map["tp"] = output_dim
                 assert any(
                     [
@@ -848,10 +847,7 @@ class ParallelTopoMapper:
                     ]
                 ), f"QKVParallelLinear {param_name} is not in packed_modules_mapping {self.weight_mapper.packed_modules_mapping}."
             elif isinstance(part, (MergedColumnParallelLinear)):
-                output_dim = getattr(param, "output_dim", None)
-                assert (
-                    output_dim is not None
-                ), f"MergedColumnParallelLinear {param_name} has no output_dim attribute."
+                output_dim = getattr(param, "output_dim", 0)
                 dims_map["tp"] = output_dim
                 assert any(
                     [
@@ -860,26 +856,20 @@ class ParallelTopoMapper:
                     ]
                 ), f"MergedColumnParallelLinear {param_name} is not in packed_modules_mapping {self.weight_mapper.packed_modules_mapping}."
             elif isinstance(part, (RowParallelLinear)):
-                input_dim = getattr(param, "input_dim", None)
+                input_dim = getattr(param, "input_dim", 1)
                 if not is_bias:
                     assert (
                         input_dim is not None
                     ), f"RowParallelLinear {param_name} has no input_dim attribute."
                     dims_map["tp"] = input_dim
             elif isinstance(part, (ColumnParallelLinear)):
-                output_dim = getattr(param, "output_dim", None)
-                assert (
-                    output_dim is not None
-                ), f"ColumnParallelLinear {param_name} has no output_dim attribute."
+                output_dim = getattr(param, "output_dim", 0)
                 dims_map["tp"] = output_dim
             elif isinstance(part, VocabParallelEmbedding):
-                output_dim = getattr(param, "output_dim", None)
+                output_dim = getattr(param, "output_dim", 0)
                 assert (
                     not is_bias
                 ), f"VocabParallelEmbedding {param_name} should not have bias."
-                assert (
-                    output_dim is not None
-                ), f"VocabParallelEmbedding {param_name} has no output_dim attribute."
                 dims_map["tp"] = output_dim
             else:
                 assert (
@@ -925,6 +915,11 @@ class ParallelTopoMapperGroup:
         self.mapper_group: List[ParallelTopoMapper] = []
 
         if weight_mapper is None:
+            if model_type not in WeightMapper._MODEL_WEIGHT_MAPPER_REGISTRY:
+                logger.warning(
+                    f"[ParallelTopoMapperGroup] can not find {model_type} in weight mapper, use {COSMOS_HF_MODEL_TYPES} model type instead."
+                )
+                model_type = COSMOS_HF_MODEL_TYPES
             weight_mapper_fn = WeightMapper.get_weight_mapper(model_type)
             self.weight_mapper = weight_mapper_fn(hf_config)
         else:
