@@ -78,7 +78,8 @@ class ncclDataTypeEnum:
     ncclFloat64 = 8
     ncclDouble = 8
     ncclBfloat16 = 9
-    ncclNumTypes = 10
+    ncclFloat8e4m3 = 10
+    ncclFloat8e5m2 = 11
 
     @classmethod
     def from_torch(cls, dtype: torch.dtype) -> int:
@@ -98,6 +99,10 @@ class ncclDataTypeEnum:
             return cls.ncclFloat64
         if dtype == torch.bfloat16:
             return cls.ncclBfloat16
+        if dtype == torch.float8_e4m3fn:
+            return cls.ncclFloat8e4m3
+        if dtype == torch.float8_e5m2:
+            return cls.ncclFloat8e5m2
         raise ValueError(f"Unsupported dtype: {dtype}")
 
 
